       										
#include <stdio.h>
#include <math.h>
#include <mach/mach.h>
#include <CoreFoundation/CFNumber.h>
#include <IOKit/IOKitLib.h>
#include <IOKit/IOCFPlugIn.h>
#include <IOKit/usb/IOUSBLib.h>

mach_port_t 	masterPort = 0;
char			outBuf[8096];
char			inBuf[8096];

void dealWithDevice(io_service_t usbDeviceRef);
void dealWithInterface(io_service_t usbInterfaceRef);
void sendRequest(IOUSBInterfaceInterface **intf, UInt16 wValue, UInt16 wIndex);
void initStageA (IOUSBInterfaceInterface **intf);
void initStageB (IOUSBInterfaceInterface **intf);
void initStream(IOUSBInterfaceInterface **intf);
void setLED (IOUSBInterfaceInterface **intf, int color);
void setChannel(IOUSBInterfaceInterface **intf, uint8_t ch);
float getSignalLevel(IOUSBInterfaceInterface **intf);


// メイン
int main (int argc, const char * argv[])
{
    kern_return_t			err;
    CFMutableDictionaryRef 	matchingDictionary = 0;		// requires <IOKit/IOKitLib.h>
    SInt32					idVendor = 0x7A69;
    SInt32					idProduct = 0x0001;
    CFNumberRef				numberRef;
    io_iterator_t			iterator = 0;
    io_service_t			usbDeviceRef;
    
    err = IOMasterPort(MACH_PORT_NULL, &masterPort);				
    if (err)
    {
        printf("USBSimpleExample: could not create master port, err = %08x\n", err);
        return err;
    }
    matchingDictionary = IOServiceMatching(kIOUSBDeviceClassName);	// requires <IOKit/usb/IOUSBLib.h>
    if (!matchingDictionary)
    {
        printf("USBSimpleExample: could not create matching dictionary\n");
        return -1;
    }
    numberRef = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &idVendor);
    if (!numberRef)
    {
        printf("USBSimpleExample: could not create CFNumberRef for vendor\n");
        return -1;
    }
    CFDictionaryAddValue(matchingDictionary, CFSTR(kUSBVendorID), numberRef);
    CFRelease(numberRef);
    numberRef = 0;
    numberRef = CFNumberCreate(kCFAllocatorDefault, kCFNumberSInt32Type, &idProduct);
    if (!numberRef)
    {
        printf("USBSimpleExample: could not create CFNumberRef for product\n");
        return -1;
    }
    CFDictionaryAddValue(matchingDictionary, CFSTR(kUSBProductID), numberRef);
    CFRelease(numberRef);
    numberRef = 0;
    
    err = IOServiceGetMatchingServices(masterPort, matchingDictionary, &iterator);
    matchingDictionary = 0;			// this was consumed by the above call
    
    while ( (usbDeviceRef = IOIteratorNext(iterator)) )
    {
		printf("Found device %p\n", (void*)usbDeviceRef);
		dealWithDevice(usbDeviceRef);
		IOObjectRelease(usbDeviceRef);			// no longer need this reference
    }
    
    IOObjectRelease(iterator);
    iterator = 0;
    
    mach_port_deallocate(mach_task_self(), masterPort);
    return 0;
}


// デバイス処理
void dealWithDevice(io_service_t usbDeviceRef)
{
    IOReturn						err;
    IOCFPlugInInterface				**iodev;		// requires <IOKit/IOCFPlugIn.h>
    IOUSBDeviceInterface245			**dev;
    SInt32							score;
    UInt8							numConf;
    IOUSBFindInterfaceRequest		interfaceRequest;
    io_iterator_t					iterator;
    io_service_t					usbInterfaceRef;
    
    err = IOCreatePlugInInterfaceForService(usbDeviceRef, kIOUSBDeviceUserClientTypeID, kIOCFPlugInInterfaceID, &iodev, &score);
    if (err || !iodev)
    {
		printf("dealWithDevice: unable to create plugin. ret = %08x, iodev = %p\n", err, iodev);
		return;
    }
    err = (*iodev)->QueryInterface(iodev, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID245), (LPVOID)&dev);
	IODestroyPlugInInterface(iodev);				// done with this

    if (err || !dev)
    {
		printf("dealWithDevice: unable to create a device interface. ret = %08x, dev = %p\n", err, dev);
		return;
    }
    err = (*dev)->USBDeviceOpen(dev);
    if (err)
    {
		printf("dealWithDevice: unable to open device. ret = %08x\n", err);
		return;
    }
    err = (*dev)->GetNumberOfConfigurations(dev, &numConf);
    if (err || !numConf)
    {
		printf("dealWithDevice: unable to obtain the number of configurations. ret = %08x\n", err);
        (*dev)->USBDeviceClose(dev);
        (*dev)->Release(dev);
		return;
    }
    printf("dealWithDevice: found %d configurations\n", numConf);
    /*err = (*dev)->GetConfigurationDescriptorPtr(dev, 0, &confDesc);			// get the first config desc (index 0)
    if (err)
    {
		printf("dealWithDevice:unable to get config descriptor for index 0\n");
        (*dev)->USBDeviceClose(dev);
        (*dev)->Release(dev);
		return;
    }*/
    /*err = (*dev)->SetConfiguration(dev, confDesc->bConfigurationValue);
    if (err)
    {
		printf("dealWithDevice: unable to set the configuration\n");
        (*dev)->USBDeviceClose(dev);
        (*dev)->Release(dev);
		return;
    }*/
    
    interfaceRequest.bInterfaceClass = kIOUSBFindInterfaceDontCare;		// requested class
    interfaceRequest.bInterfaceSubClass = kIOUSBFindInterfaceDontCare;		// requested subclass
    interfaceRequest.bInterfaceProtocol = kIOUSBFindInterfaceDontCare;		// requested protocol
    interfaceRequest.bAlternateSetting = kIOUSBFindInterfaceDontCare;		// requested alt setting
    
    err = (*dev)->CreateInterfaceIterator(dev, &interfaceRequest, &iterator);
    if (err)
    {
		printf("dealWithDevice: unable to create interface iterator\n");
        (*dev)->USBDeviceClose(dev);
        (*dev)->Release(dev);
		return;
    }
    
    /*while ( (usbInterfaceRef = IOIteratorNext(iterator)) )
    {
		printf("found interface: %p\n", (void*)usbInterfaceRef);
		dealWithInterface(usbInterfaceRef);
		IOObjectRelease(usbInterfaceRef);				// no longer need this reference
    }*/
	usbInterfaceRef = IOIteratorNext(iterator);
	dealWithInterface(usbInterfaceRef);
	IOObjectRelease(usbInterfaceRef);
    
    IOObjectRelease(iterator);
    iterator = 0;

    err = (*dev)->USBDeviceClose(dev);
    if (err)
    {
		printf("dealWithDevice: error closing device - %08x\n", err);
		(*dev)->Release(dev);
		return;
    }
    err = (*dev)->Release(dev);
    if (err)
    {
		printf("dealWithDevice: error releasing device - %08x\n", err);
		return;
    }
}


// インターフェース処理
void dealWithInterface(io_service_t usbInterfaceRef)
{
    IOReturn					err;
    IOCFPlugInInterface 		**iodev;		// requires <IOKit/IOCFPlugIn.h>
    IOUSBInterfaceInterface 	**intf;
    SInt32						score;

    err = IOCreatePlugInInterfaceForService(usbInterfaceRef, kIOUSBInterfaceUserClientTypeID, kIOCFPlugInInterfaceID, &iodev, &score);
    if (err || !iodev)
    {
		printf("dealWithInterface: unable to create plugin. ret = %08x, iodev = %p\n", err, iodev);
		return;
    }
    err = (*iodev)->QueryInterface(iodev, CFUUIDGetUUIDBytes(kIOUSBInterfaceInterfaceID245), (LPVOID)&intf);
	IODestroyPlugInInterface(iodev);				// done with this
	
    if (err || !intf)
    {
		printf("dealWithInterface: unable to create a device interface. ret = %08x, intf = %p\n", err, intf);
		return;
    }
	
    err = (*intf)->USBInterfaceOpen(intf);
    if (err)
    {
		printf("dealWithInterface: unable to open interface. ret = %08x\n", err);
		return;
    }
	
	// ===========================================================================
	printf("================================\n");
	// 初期化A
	initStageA(intf);
	printf("================================\n");
	// 初期化B
	initStageB(intf);
	printf("================================\n");
	// ストリーム初期化
	initStream(intf);
	printf("================================\n");
	setLED(intf, 1);
	// チャンネル設定
	setChannel(intf, 25); // とりあえず日本テレビ
	// シグナルレベル取得
	float signal1 = getSignalLevel(intf);
	printf("Signal level: %f\n", signal1);
	
	// チャンネル設定
	setChannel(intf, 21); // ついでにフジテレビ
	// シグナルレベル取得
	float signal2 = getSignalLevel(intf);
	printf("Signal level: %f\n", signal2);
	printf("================================\n");
	printf("Set LED\n");
	// LEDを紫、緑、赤に順に変更
	setLED(intf, 0);
	sleep(5);
	setLED(intf, 1);
	sleep(5);
	setLED(intf, 2);
	// ===========================================================================

	err = (*intf)->USBInterfaceClose(intf);
    if (err)
    {
		printf("dealWithInterface: unable to close interface. ret = %08x\n", err);
		return;
    }
	
    err = (*intf)->Release(intf);
    if (err)
    {
		printf("dealWithInterface: unable to release interface. ret = %08x\n", err);
		return;
    }
}


// リクエスト送信関数
void sendRequest(IOUSBInterfaceInterface **intf, UInt16 wValue, UInt16 wIndex) 
{
	unsigned char iobuff[32];
	
	IOUSBDevRequest req;
	req.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBVendor, kUSBDevice);
	req.bRequest = 0x01;
	req.wValue = wValue;
	req.wIndex = wIndex;
	req.wLength = 0;
	req.wLenDone = 0;
	req.pData = iobuff;
	
	IOReturn err = (*intf)->ControlRequest(intf, 0, &req);
	if(err) {
		printf("ERROR: %X\n", err);
	}
}


// 初期化処理A
void initStageA (IOUSBInterfaceInterface **intf)
{
	sendRequest(intf, 0x0002, 0x0011);
	sendRequest(intf, 0x0000, 0x0011);
	
	unsigned char iobuff1[32];
	IOUSBDevRequest req1;
	req1.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req1.bRequest = 0x02;
	req1.wValue = 0xA000;
	req1.wIndex = 0x00FE;
	req1.wLength = 1;
	req1.wLenDone = 0;
	req1.pData = iobuff1;
	IOReturn err1 = (*intf)->ControlRequest(intf, 0, &req1);
	if(err1) {
		printf("ERROR: %X\n", err1);
	}

	unsigned char iobuff2[32];
	IOUSBDevRequest req2;
	req2.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req2.bRequest = 0x02;
	req2.wValue = 0x3000;
	req2.wIndex = 0x0080;
	req2.wLength = 1;
	req2.wLenDone = 0;
	req2.pData = iobuff2;
	IOReturn err2 = (*intf)->ControlRequest(intf, 0, &req2);
	if(err2) {
		printf("ERROR: %X\n", err2);
	}
	
	sendRequest(intf, 0x0004, 0x0030);
	printf("Init Stage A complete.\n");
}


// 初期化処理B
void initStageB (IOUSBInterfaceInterface **intf)
{
	sendRequest(intf, 0x3040, 0x01);
	sendRequest(intf, 0x3038, 0x04);
	sendRequest(intf, 0x3040, 0x05);
	sendRequest(intf, 0x3040, 0x07);
	sendRequest(intf, 0x304F, 0x0F);
	sendRequest(intf, 0x3021, 0x11);
	sendRequest(intf, 0x300B, 0x12);
	sendRequest(intf, 0x302F, 0x13);
	sendRequest(intf, 0x3031, 0x14);
	sendRequest(intf, 0x3002, 0x16);
	sendRequest(intf, 0x30C4, 0x21);
	sendRequest(intf, 0x3020, 0x22);
	sendRequest(intf, 0x3079, 0x2C);
	sendRequest(intf, 0x3034, 0x2D);
	sendRequest(intf, 0x3000, 0x2F);
	sendRequest(intf, 0x3028, 0x30);
	sendRequest(intf, 0x3031, 0x31);
	sendRequest(intf, 0x30DF, 0x32);
	sendRequest(intf, 0x3001, 0x38);
	sendRequest(intf, 0x3078, 0x39);
	sendRequest(intf, 0x3033, 0x3B);
	sendRequest(intf, 0x3033, 0x3C);
	sendRequest(intf, 0x3090, 0x48);
	sendRequest(intf, 0x3068, 0x51);
	sendRequest(intf, 0x3038, 0x5E);
	sendRequest(intf, 0x3000, 0x71);
	sendRequest(intf, 0x3008, 0x72);
	sendRequest(intf, 0x3000, 0x77);
	sendRequest(intf, 0x3021, 0xC0);
	sendRequest(intf, 0x3010, 0xC1);
	sendRequest(intf, 0x301A, 0xE4);
	sendRequest(intf, 0x301F, 0xEA);
	sendRequest(intf, 0x3000, 0x77);
	sendRequest(intf, 0x3000, 0x71);
	sendRequest(intf, 0x3000, 0x71);
	sendRequest(intf, 0x300C, 0x76);
	
	printf("Init Stage B complete.\n");
}


// ストリーム初期化関数
void initStream(IOUSBInterfaceInterface **intf) 
{
	send(intf, 0x08, 0x33);
	send(intf, 0x40, 0x37);
	send(intf, 0x1F, 0x3A);
	send(intf, 0xFF, 0x3B);
	send(intf, 0x1F, 0x3C);
	send(intf, 0xFF, 0x3D);
	send(intf, 0x00, 0x38);
	send(intf, 0x00, 0x35);
	send(intf, 0x00, 0x39);
	send(intf, 0x00, 0x36);
	
	printf("Init Stream complete.\n");
}


// LED設定関数
void setLED(IOUSBInterfaceInterface **intf, int color)
{
	sendRequest(intf, 0x000F, 0x06);
	switch(color) {
		// 紫
		case 0:
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0009, 0x00);
			sendRequest(intf, 0x000D, 0x00);
			break;
		// 緑
		case 1:
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0001, 0x00);
			sendRequest(intf, 0x0005, 0x00);
			break;
		// 赤
		case 2:
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x000B, 0x00);
			sendRequest(intf, 0x000F, 0x00);
			sendRequest(intf, 0x0003, 0x00);
			sendRequest(intf, 0x0007, 0x00);
			sendRequest(intf, 0x0001, 0x00);
			sendRequest(intf, 0x0005, 0x00);
			break;
		// 白
		case 3:
			// 情報なし
			break;
	}
	sendRequest(intf, 0x0000, 0x06);
}


// チャンネルの設定
void setChannel(IOUSBInterfaceInterface **intf, uint8_t ch)
{
	// チャンネルのチェック
	if(ch < 13 || ch > 62) {
		printf("Invalid channel.\n");
	}
	
	// チャンネルデータ計算
	const uint32_t chCode = 0x0E7F + 0x002A * (uint32_t)(ch - 13);
	
	// リクエスト用バッファ
	uint8_t sbuf[2][5];
	memset(sbuf, '\0', sizeof(sbuf));
	uint8_t rbuf[2][1];
	memset(rbuf, '\0', sizeof(rbuf));
	
	// 1回目リクエスト設定	
	sbuf[0][0] = 0xC0;
	sbuf[0][1] = (uint8_t)(chCode >> 8);
	sbuf[0][2] = (uint8_t)(chCode & 0xFF);
	sbuf[0][3] = 0xB2;
	sbuf[0][4] = 0x08;
	
	// 2回目リクエスト設定
	sbuf[1][0] = 0xC0;
	sbuf[1][1] = (uint8_t)(chCode >> 8);
	sbuf[1][2] = (uint8_t)(chCode & 0xFF);
	sbuf[1][3] = 0x9A;
	sbuf[1][4] = 0x50;
	
	// 1回目のリクエスト送信
	IOUSBDevRequest req1;
	req1.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBVendor, kUSBDevice);
	req1.bRequest = 0x03;
	req1.wValue = 0x3000;
	req1.wIndex = 0x00FE;
	req1.wLength = sizeof(sizeof(sbuf[0]));
	req1.wLenDone = 0;
	req1.pData = sbuf[0];
	IOReturn err1 = (*intf)->ControlRequest(intf, 0, &req1);
	if(err1) {
		printf("ERROR(1): %X\n", err1);
	}
	
	// 2回目のリクエスト送信
	IOUSBDevRequest req2;
	req2.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBVendor, kUSBDevice);
	req2.bRequest = 0x03;
	req2.wValue = 0x3000;
	req2.wIndex = 0x00FE;
	req2.wLength = sizeof(sizeof(sbuf[1]));
	req2.wLenDone = 0;
	req2.pData = sbuf[1];
	IOReturn err2 = (*intf)->ControlRequest(intf, 0, &req2);
	if(err2) {
		printf("ERROR(2): %X\n", err2);
	}	
	
	// 3回目のリクエスト送信
	IOUSBDevRequest req3;
	req3.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req3.bRequest = 0x02;
	req3.wValue = 0x3000;
	req3.wIndex = 0x00B0;
	req3.wLength = sizeof(sizeof(rbuf[0]));
	req3.wLenDone = 0;
	req3.pData = rbuf[0];
	IOReturn err3 = (*intf)->ControlRequest(intf, 0, &req3);
	if(err3) {
		printf("ERROR(3): %X\n", err3);
	}
	
	// 4回目のリクエスト送信
	IOUSBDevRequest req4;
	req4.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req4.bRequest = 0x02;
	req4.wValue = 0x3000;
	req4.wIndex = 0x0080;
	req4.wLength = sizeof(sizeof(rbuf[1]));
	req4.wLenDone = 0;
	req4.pData = rbuf[1];
	IOReturn err4 = (*intf)->ControlRequest(intf, 0, &req4);
	if(err4) {
		printf("ERROR(4): %X\n", err4);
	}
	
	printf("Set channel: %dch\n", ch);
}


// シグナルレベルの取得
float getSignalLevel(IOUSBInterfaceInterface **intf)
{
	float sig_level = 0;
	uint8_t sigbuf[37];
	uint32_t dwHex;
	memset(sigbuf, '\0', sizeof(sigbuf));

	IOUSBDevRequest req;
	req.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice);
	req.bRequest = 0x02;
	req.wValue = 0x3000;
	req.wIndex = 0x0089;
	req.wLength = sizeof(sigbuf);
	req.wLenDone = 0;
	req.pData = sigbuf;
	
	IOReturn err = (*intf)->ControlRequest(intf, 0, &req);
	if(err) {
		printf("ERROR: %X\n", err);
	}

	dwHex = ((uint32_t)(sigbuf[2]) << 16) | ((uint32_t)(sigbuf[3]) << 8) | (uint32_t)(sigbuf[4]);
	if(!dwHex) {
		sig_level = 0.0f;
	} else {
		sig_level = (float)(68.4344993f - log10((double)dwHex) * 10.0 + pow(log10((double)dwHex) - 5.045f, (double)4.0f) / 4.09f);
	}
	
	return  sig_level;
}

